<?php

require_once(dirname(__FILE__)."/".str_repeat("../",4)."lib-tk/shield.php");
require_once("lib-action/action.php");

class FirstTimePerformAction extends MB_Action
{
	public function execute()
	{

		parent::execute();

		dbg($_REQUEST);

		$action = bgpc("a");
		switch ($action) {
			case "company":
				$this->processCompany();
				break;
			case "registrar":
				$this->processRegistrar();
				break;
			case "sharedip":
				$this->processSharedip();
				break;
			case "gateway":
				$this->processGateway();
				break;
			case "taxes":
				$this->processTaxes();
				break;
			case "fraudguardian":
				$this->processFraudguardian();
				break;
			case "encryption":
				$this->processEncryption();
				break;
			default:
				redirect("configs.php", "FirstTimeIntroduction");
				break;
		}
		return $this->hasExecutedSuccessfully();
	}

	private function processCompany()
	{
		$companySettings = array(
			"company_name", "company_short_description", "company_description",
			"company_url", "company_client_url", "company_order_url", "company_address_1",
			"company_email", "company_address_2", "company_city", "company_state",
			"company_zip", "countries_id", "company_phone1",
			"company_phone2", "company_fax", "company_tax_id_number"
		);

		try {

			require_once("object_managers/CompanyManager.php");
			$company = CompanyManager::getProviderCompany();
			foreach ($companySettings as $setting) {
				$company->$setting = bgpc($setting);
			}
			CompanyManager::update($company);
			redirect("configs.php", "FirstTimeDisplayStep", array(
				"a" => "registrar",
			));
		} catch (ProductException $e) {
			$this->addOperationErrors($e);
			$this->addErrorTemplates();
			return $this->hasExecutedSuccessfully();
		}
	}

	private function processRegistrar()
	{
		try {
			$registrarConfigParams = bgpc("registrars", array());
			$registrars = Factory()->ConfigGroup(0)->getChild("domain")->getChild("registrar")->ConfigGroupList();
			require_once("ModuleTool.php");
			$chosenRegistrars = array();
			foreach($registrars as $registrar) {
				$input_form = UIFactory()->Form();
				$controls = array();
				foreach ( ModuleTool::getModuleControls("registrar", $registrar->config_group_name) as $inputPanel ) {
					$inputPanel->initFromArray(DataObject::getMapOf($registrar->ConfigParams, "name", "value"));
					foreach ( $inputPanel->getInputControls() as $input_control ) {
						$inputControlName = $input_control->name;
						$input_control->name = "registrars_{$registrar->config_group_id}_{$input_control->name}";
						$controls[$inputControlName] = $input_control;
					}
					$input_form->addInputPanel($inputPanel);
				}
				$input_form->initFromRequest();

				if(isset($_REQUEST["registrars_{$registrar->config_group_id}_moduleEnabled"])) {
					$error_messages = array();
					if ( !$input_form->validate($error_messages) ) {
						throw new ProductException(ProductException::E_OBJECT_HAS_INVALID_ATTRS, "Invalid input for '{$registrar->config_group_name}' module:<br>".implode("<br>", $error_messages));
					}
				}

				$registrar->ConfigParams->clear();
				foreach ( $controls as $paramName => $input_control ) {
					if("moduleEnabled" == $paramName) {
						if(1 == $input_control->value) {
							$chosenRegistrars[] = $registrar->config_group_id;
						}
					} elseif("modulePassword" == $paramName) {
						if("" != $input_control->value) {
							$registrar->ConfigParams[$paramName]->value = getEncryptedText(getStuff($input_control->value));
						} else {
							$registrar->ConfigParams[$paramName]->value = "";
						}
					} else {
						$registrar->ConfigParams[$paramName]->value = $input_control->value;
					}
				}
				$registrar->ConfigParams->update();
			}

			$tldsToSet = array();
			foreach ($registrars as $registrar) {
				if (in_array($registrar->config_group_id, $chosenRegistrars)) {
					$registrar->ConfigParams["moduleEnabled"]->value = 1;
					$tldsToSet += $this->getRegistrarTLDsToSet($registrar->config_group_id);
				} else {
					$registrar->ConfigParams["moduleEnabled"]->value = 0;
				}
				$registrar->ConfigParams->update();
			}

			// set correct tlds values
			foreach (Factory()->TLDList() as $tld) {
				if (!isset($tldsToSet[$tld->tld_name])) {
					$tld->registrar_id = 0;
					$tld->tld_active = 0;
				} else {
					$tld->tld_active = 1;
					$tld->registrar_id = $tldsToSet[$tld->tld_name];
				}
				$tld->update();
			}
			
			redirect("configs.php", "FirstTimeDisplayStep", array(
				"a" => "sharedip",
			));

		} catch (ProductException $e) {
			$this->addOperationErrors($e);
			$this->addErrorTemplates();
			return $this->hasExecutedSuccessfully();
		}
	}

	private function processGateway()
	{
		$configured_gateways = array();
		foreach ( Factory()->PaymentGatewayList() as $gateway ) {
			$configured_gateways["{$gateway->payment_gateway_module}{$gateway->id}"] = $gateway;
		}

		try {
			$primary_gateway_id = null;
			$primary_gateways = bgpc("pgateways_e");
			if ( !empty($primary_gateways) ) {
				$primary_gateway = $this->updatePaymentGateway($primary_gateways[0], $configured_gateways);
				$primary_gateway_id = $primary_gateway->id;
			}

			$third_party_gateway_ids = array();
			foreach ( bgpc("sgateways_e", array()) as $config_name ) {
				$third_party_gateway = $this->updatePaymentGateway($config_name, $configured_gateways);
				$third_party_gateway_ids[] = $third_party_gateway->id;
			}

			require_once("object_managers/OrderFormManager.php");
			$default_form = OrderFormManager::getDefaultForm();
			$default_form->payment_gateway_id = $primary_gateway_id;
			$default_form->thirdPartyGatewayIDs = $third_party_gateway_ids;
			OrderFormManager::update($default_form);
			redirect("configs.php", "FirstTimeDisplayStep", array("a" => "taxes"));
		} catch (ProductException $e) {
			$this->addOperationErrors($e);
			$this->addErrorTemplates();
			return $this->hasExecutedSuccessfully();
		}
	}

	private function processSharedip()
	{
		updateForKey(":configParams", $_REQUEST["configParamID"], array(
			"configParamValue" => $_REQUEST["sharedip"],
		));
		redirect("configs.php", "FirstTimeDisplayStep", array(
			"a" => "gateway",
		));
		exit;
	}

	private function processTaxes()
	{
		$taxZoneGroupID = bgpc("taxZoneGroupID");
		$taxZoneID = bgpc("taxZoneID");

		if ("" == $taxZoneGroupID){
			$taxZoneGroupID = insertFor(":taxZoneGroups", array(
				"taxZoneGroupName" => "Products",
				"taxZoneGroupActive" => 1,
			));
		}

		require_once("object_managers/CompanyManager.php");
		if ("" == $taxZoneID){
			require_once mb_path("/lib-billing/include/accounting/Accounting.php");
			$taxZoneID = insertFor(":taxZones", array(
				"countriesISO2" => CompanyManager::getProviderCompany()->Country->countries_iso_2,
				"taxZoneName" => "*",
				"taxZoneAmountType" => 0,
				"taxZoneAmount" => $_REQUEST["taxZoneAmount"],
				"taxZoneDescription" => $_REQUEST["taxZoneDescription"],
				"taxZoneActive" => 1,
				"ledgerAccountID" => getGeneralLedgerAccountID("taxes_payable"),
			));

			DatabaseManager::getConnection()->exec("
				INSERT INTO tax_zone_group_zone_rel (tax_zone_group_id, tax_zone_id)
				VALUES ($taxZoneGroupID, $taxZoneID)
			");
		} else {
			updateForKey(":taxZones", $taxZoneID, array(
				"taxZoneAmount" => $_REQUEST["taxZoneAmount"],
			));
		}

		$variants = arrayFor(":productVariants", array(
			"taxZoneGroupID" => $taxZoneGroupID,
		));

		foreach ((array)$variants as $variant){
			updateForKey(":productVariants", $variant["productVariantID"], array(
				"taxZoneGroupID" => 0,
			));
		}

		if ((int)$_REQUEST["enableTaxes"]){
			foreach ((array)$_REQUEST["taxableProducts"] as $variantID){
				updateForKey(":productVariants", $variantID, array(
					"taxZoneGroupID" => $taxZoneGroupID,
				));
			}
		}

		require_once("object_managers/OrderFormManager.php");
		try {
			$order_form = OrderFormManager::getDefaultForm();
			$order_form->default_tax_zone_id = is_null(bgpc("defaultGrossPriceDisplay")) ? NULL : $taxZoneID;
			$order_form->request_tax_vat_id = is_null(bgpc("requestTaxVatID")) ? 0 : 1 ;
			$order_form->order_form_verify_vat = is_null(bgpc("orderFormVerifyVat")) ? 0 : 1 ;
			OrderFormManager::update($order_form);
			redirect("configs.php", "FirstTimeDisplayStep", array(
				"a" => "encryption",
			));
		} catch (ProductException $e) {
			$this->addOperationErrors($e);
			$this->addErrorTemplates();
			return $this->hasExecutedSuccessfully();
		}
		exit;
	}

	private function processEncryption()
	{
		try {
			$encryption_settings = array();
			$encryption_settings["encryption_agree"] = ($encryption_agree = bgpc("enableEncryption"));
			require_once("object_managers/CryptManager.php");
			if ( "agree" == $encryption_agree ) {
				$lek_pin = bgpc("lek_pin");
				if($lek_pin != bgpc("lek_pin_confirm")) {
					throw new InvalidAttributes(array("lek_pin", "lek_pin_confirm"));
				}
				$encryption_hash = CryptManager::generateEncryptionHash($lek_pin);
				$encryption_settings["lek_pin"] = $lek_pin;
				$encryption_settings["encryption_hash"] = $encryption_hash;
			}
			CryptManager::update($encryption_settings);
			redirect("configs.php", "FirstTimeDisplayStep", array("a" => "fraudguardian"));
		} catch (ProductException $e) {
			$this->addOperationErrors($e);
			$this->addErrorTemplates();
			return $this->hasExecutedSuccessfully();
		}
	}

	private function processFraudguardian()
	{
		$fraudguardian = Factory()->ConfigGroup(0)->getChild("securitySettings")->getChild("fraudguardian")->getChild("0");
		$fraudguardianParams["enableFraudGuardian"] = is_null(bgpc("enableFraudGuardian")) ? 0 : 1 ;
		$fraudguardianParams["usernameFraudGuardian"] = bgpc("usernameFraudGuardian");
		$fraudguardianParams["passwordFraudGuardian"] = bgpc("passwordFraudGuardian");
		foreach ($fraudguardianParams as $paramName => $paramValue) {
			$fraudguardian->ConfigParams[$paramName]->value = $paramValue;
		}
		$fraudguardian->ConfigParams->update();

		global $CONF;
		$threshold = oneOf(":fraudCountryThresholds");
		if (!(int)$threshold["thresholdID"]){
			$thresholdID = insertFor(":fraudCountryThresholds", array(
				'thresholdName' => 'default',
				'countryType' => '0',
				'countriesID' => '225',
				'regionsID' => '002',
				'levelOneScore' => '1.0',
				'levelTwoScore' => '3.0',
				'levelThreeScore' => '5.0',
				'levelOneAction' => '0',
				'levelTwoAction' => '2',
				'levelThreeAction' => '3',
				'catchAllAction' => '5',
				'overrideOneAmount' => '0.00',
				'overrideTwoAmount' => '0.00',
				'overrideThreeAmount' => '0.00',
				'overrideOneAction' => '0',
				'overrideTwoAction' => '0',
				'overrideThreeAction' => '0',
				'primaryScoreToUse' => '0',
				'freeEmailWeight' => '2.5',
				'countryMatchWeight' => '2.5',
				'anonProxyWeight' => '5.0',
				'countryRiskWeight' => '5.0',
				'ipDistanceWeight' => '10.0',
				'binMatchWeight' => '2.0',
				'binBankMatchWeight' => '1.0',
				'highRiskEmailWeight' => '5.0',
				'highRiskShippingWeight' => '5.0',
				'openProxyWeight' => '2.5',
				'fraudSoftRejectMessage' => '',
				'fraudHardRejectMessage' => '',
			));

			$fraudSetID = insertFor(":fraudSets", array(
				"fraudSetName" => "Default",
				"fraudSetEmail" => $CONF["user"]["adminData"]["adminEmail"],
			));

			insertFor(":fraudSetThresholdRels", array(
				"thresholdID" => $thresholdID,
				"fraudSetID" => $fraudSetID,
			));

			$orderForm = oneOf(":orderForms");
			$orderFormID = $orderForm["orderFormID"];

			insertFor(":fraudSetOrderFormRels", array(
				"fraudSetID" => $fraudSetID,
				"orderFormID" => $orderFormID,
			));

		}

		$CONF["system"]["internal"]["suppressFirstTimeWizard"] = 1;
		storeConfigs("system", "internal");

		redirect("configs.php", "FirstTimeDisplayStep", array(
			"a" => "finished",
		));
		exit;
	}



	private function updatePaymentGateway($config_name, $configured_gateways)
	{
		$module_name = isset($configured_gateways[$config_name]) ? $configured_gateways[$config_name]->payment_gateway_module : $config_name;
		$input_form = UIFactory()->Form();
		$input_panel = UIFactory()->InputPanel("");
		require_once("object_managers/PaymentGatewayManager.php");
		$input_gateway_name = UIFactory()->InputString("{$config_name}_payment_gateway_name", blmsg("TRANS_GATEWAY_NAME"), true,
			array("PaymentGatewayManager", "checkName", isset($configured_gateways[$config_name]) ? $configured_gateways[$config_name]->id : null)
		);
		$input_panel->addInputControl($input_gateway_name);
		$input_form->addInputPanel($input_panel);
		$module_input_controls = array();
		require_once("ModuleTool.php");
		$module_controls = ModuleTool::getModuleControls("gateway", $module_name);
		foreach ( $module_controls as $control ) {
			foreach ( $control->getInputControls() as $input_control ) {
				$input_control->name = "{$config_name}_{$input_control->name}";
				$module_input_controls[] = $input_control;
			}
			$input_form->addInputPanel($control);
		}

		$input_form->initFromRequest();
		$error_messages = array();
		if ( !$input_form->validate($error_messages) ) {
			$module_info = PaymentGatewayManager::getModuleInfo($module_name);
			throw new ProductException(ProductException::E_OBJECT_HAS_INVALID_ATTRS, "Invalid input for '{$module_info['displayName']}' module:<br>".implode("<br>", $error_messages));
		}

		if ( !isset($configured_gateways[$config_name]) ) {
			$module_info = PaymentGatewayManager::getModuleInfo($module_name);
			$gateway = Factory()->PaymentGateway();
			$gateway->payment_gateway_module = $module_name;
			$gateway->payment_gateway_type = $module_info["gatewaytype"];
		} else {
			$gateway = $configured_gateways[$config_name];
		}
		$gateway->payment_gateway_name = $input_gateway_name->value;

		$gateway->PaymentGatewayParams->clear();
		foreach ( $module_input_controls as $input_control ) {
			$param_name = substr($input_control->name, strlen($config_name) + 1);
			$gateway->PaymentGatewayParams[$param_name]->value = $input_control->value;
		}

		PaymentGatewayManager::update($gateway);
		return $gateway;
	}

	// get registrar tlds which can be set and works properly
	private function getRegistrarTLDsToSet($registrarID)
	{
		require_once("ModuleTool.php");
		$registrar = ModuleTool::getModuleInstance("registrar", Factory()->ConfigGroup($registrarID)->config_group_name, array('configGroupID'=> $registrarID));
		if (!isset($registrar->capabilities[REGISTRAR_TLDS])) {
			return array();
		}

		return array_fill_keys($registrar->capabilities[REGISTRAR_TLDS], $registrarID);
	}
}
