<?php

require_once(dirname(__FILE__)."/".str_repeat("../",4)."lib-tk/shield.php");
require_once("lib-action/DomainAction.php");
require_once("lib-mbapi/mbapi.php");

/**
 * This Action object takes information provided by the EditDomainContactData action, applies that information to
 * records in the database, and redirects to the ShowDomainContactDetails action for the given domain.
 *
 * @author  Michael Fountain
 */
class EditDomainContactDataResponse extends DomainAction {
	/**
	 * The constructor for an EditDomainContactDataResponse action object. This merely calls the parent constructor,
	 * and sets the string representation of this object. It also sets up $this->params.
	 */
	public function __construct()
	{
		parent::__construct();
		$this->params = array
		(
			"domainID",

			"registrantAddress1", "registrantAddress2", "registrantCity", "registrantCountry",
			"registrantEmailAddress", "registrantFax", "registrantFirstName", "registrantJobTitle",
			"registrantLastName", "registrantOrganizationName", "registrantPhone", "registrantPhoneExt",
			"registrantPostalCode", "registrantStateProvince", "registrantStateProvinceChoice",

			"auxAddress1", "auxAddress2", "auxCity", "auxCountry", "auxEmailAddress", "auxFax",
			"auxFirstName", "auxJobTitle", "auxLastName", "auxOrganizationName", "auxPhone",
			"auxPhoneExt", "auxPostalCode", "auxStateProvince", "auxStateProvinceChoice",

			"techAddress1", "techAddress2", "techCity", "techCountry", "techEmailAddress",
			"techFax", "techFirstName", "techJobTitle", "techLastName", "techOrganizationName",
			"techPhone", "techPhoneExt", "techPostalCode", "techStateProvince",
			"techStateProvinceChoice",

			"adminAddress1", "adminAddress2", "adminCity", "adminCountry", "adminEmailAddress",
			"adminFax", "adminFirstName", "adminJobTitle", "adminLastName", "adminOrganizationName",
			"adminPhone", "adminPhoneExt", "adminPostalCode", "adminStateProvince",
			"adminStateProvinceChoice",

			"billingPartyID", "billingAddress1", "billingAddress2", "billingCity",
			"billingCountry", "billingEmailAddress", "billingFax", "billingFirstName",
			"billingJobTitle", "billingLastName", "billingOrganizationName", "billingPhone",
			"billingPhoneExt", "billingPostalCode", "billingStateProvince", "billingStateProvinceChoice",
			"billingFullCountry", "useBelowAuxInfo"
		);
	}

	/**
	 * The main method of this class that executes the EditDomainResponse action, and returns success or failure.
	 *
	 * @return  1 if this action executed successfully, or 0 if otherwise.
	 */
	function execute() {
		global $CONF;
		parent::execute();

	 	$mbapiData = $this->getDomain();

		// fetching domain data from the registrar
		$domainSLD = $mbapiData["mbapi"][0]["results"][0]["domains"][0]["domain"][0]["domainSLD"][0];
		$domainTLD = $mbapiData["mbapi"][0]["results"][0]["domains"][0]["domain"][0]["domainTLD"][0];
		$domainID = $mbapiData["mbapi"][0]["results"][0]["domains"][0]["domain"][0]["domainID"][0];

		$contactXML = "\n";
		$contactData = array("registrant","admin","tech","auxBilling");

		foreach($contactData as $contactType) {

			// clean up state/province data
			if (eregi("S",$_REQUEST[$contactType]["StateProvinceChoice"])) {
				$_REQUEST[$contactType]["StateProvince"] = $_REQUEST[$contactType]["State"];
			} else if (eregi("P",$_REQUEST[$contactType]["StateProvinceChoice"])) {
				$_REQUEST[$contactType]["StateProvince"] = $_REQUEST[$contactType]["Province"];
			} else {
				$_REQUEST[$contactType]["StateProvince"] = "";
			}

			// remove variables not needed by registrars
			unset($_REQUEST[$contactType]["State"],$_REQUEST[$contactType]["Province"]);

			// build XML command
			if (is_array($_REQUEST[$contactType])) {
			 	$contactXML .= "                              <$contactType>\n";
				foreach($_REQUEST[$contactType] as $key => $value) {
					$contactXML .= "                                <$contactType$key>$value</$contactType$key>\n";
				}
				$contactXML .=	"                              </$contactType>\n";
			}
		}
		$contactXML .= "\n";

		$xml = "
		<mbapi>
		  <command>ProcessRegistrar</command>
		  <showXMLHeader>0</showXMLHeader>
		  <params>
			<subCommand>setDomainContactData</subCommand>
			<configGroupName>0</configGroupName>
			<configID>0</configID>
			<domainID>".$domainID."</domainID>
			<domainSLD>".$domainSLD."</domainSLD>
			<domainTLD>".$domainTLD."</domainTLD>
			<domainContactData>$contactXML
			</domainContactData>
		</params>
		</mbapi>";
		$data = dispatchMBAPI($xml,1);
		$this->processRegistrarResponse($data);

		return $this->hasExecutedSuccessfully();

	} // execute

} // EditDomainResponse
?>