<?php
require_once(dirname(__FILE__)."/../../../../lib-action/action.php");
require_once("UI/Page.php");
require_once("UI/PageController.php");

class PaymentGatewaySelectModulePage extends UIPage
{
	public function createControls()
	{
		$this->input_gateway_type = UIFactory()->InputRadio("payment_gateway_type", blmsg("TRANS_GATEWAY_TYPE"));
		require_once("objects/PaymentGateway.php");
		$this->input_gateway_type->value = PaymentGateway::TYPE_PRIMARY;
		require_once("object_managers/PaymentGatewayManager.php");
		$this->input_gateway_type->addOption(
			PaymentGateway::TYPE_PRIMARY,
			blmsg(PaymentGateway::typeName(PaymentGateway::TYPE_PRIMARY)),
			blmsg("TRANS_PRIMARY_GATEWAY_DESCRIPTION")
		);

		if ($this->isHSPCAvailable()) {
			require_once("objects/PaymentGateway.php");
			$this->input_gateway_type->addOption(
				PaymentGateway::TYPE_HSPC_CREDIT_CARD,
				blmsg(PaymentGateway::typeName(PaymentGateway::TYPE_HSPC_CREDIT_CARD)),
				blmsg("TRANS_HSPC_CREDIT_CARD_GATEWAY_DESCRIPTION")
			);
			$this->input_gateway_type->addOption(
				PaymentGateway::TYPE_HSPC_THIRD_PARTY,
				blmsg(PaymentGateway::typeName(PaymentGateway::TYPE_HSPC_THIRD_PARTY)),
				blmsg("TRANS_HSPC_THIRD_PARTY_GATEWAY_DESCRIPTION")
			);
			$this->input_gateway_type->addOption(
				PaymentGateway::TYPE_HSPC_BANK_TRANSFER,
				blmsg(PaymentGateway::typeName(PaymentGateway::TYPE_HSPC_BANK_TRANSFER)),
				blmsg("TRANS_HSPC_BANK_TRANSFER_GATEWAY_DESCRIPTION")
			);
		}

		$this->input_gateway_type->addOption(
			PaymentGateway::TYPE_THIRD_PARTY,
			blmsg(PaymentGateway::typeName(PaymentGateway::TYPE_THIRD_PARTY)),
			blmsg("TRANS_THIRD_PARTY_GATEWAY_DESCRIPTION")
		);

		require_once("UI/PageController.php");
		require_once("UI/Form.php");
		$this->input_gateway_type->onclick = "submit_form('".UIForm::DEFAULT_NAME."', '".UIPageController::CMD_RELOAD."');";

		$this->input_module = UIFactory()->InputDropDown("payment_gateway_module", blmsg("TRANS_GATEWAY_MODULE"));

		$input_gateway_name = UIFactory()->InputString("payment_gateway_name", blmsg("TRANS_GATEWAY_NAME"), true,
			array("PaymentGatewayManager", "checkName")
		);
		$input_gateway_desc = UIFactory()->InputText("payment_gateway_description", blmsg("TRANS_GATEWAY_DESCRIPTION"));

		$input_panel = UIFactory()->InputPanel(blmsg("TRANS_PAYMENT_GATEWAY"));
		$input_panel->addInputControl($this->input_gateway_type);
		$input_panel->addInputControl($this->input_module);
		$input_panel->addInputControl($input_gateway_name);
		$input_panel->addInputControl($input_gateway_desc);

		$this->addInputPanel($input_panel);
	}

	public function initControl()
	{
		require_once("object_managers/PaymentGatewayManager.php");

		switch ($this->input_gateway_type->value)
		{
			case PaymentGateway::TYPE_HSPC_CREDIT_CARD:
			case PaymentGateway::TYPE_HSPC_BANK_TRANSFER:
			case PaymentGateway::TYPE_HSPC_THIRD_PARTY:
				if ($this->isHSPCAvailable()) {
					foreach (PaymentGatewayManager::getHSPCModules($this->input_gateway_type->value) as $module_opts) {
						$this->input_module->addOption($module_opts["name"], $module_opts["name"]);
					}
				}
				break;
			default:
				$statuses = PaymentGateway::statuses();
				foreach ( PaymentGatewayManager::getModules($this->input_gateway_type->value) as $module_name ) {
					$module_info = PaymentGatewayManager::getModuleInfo($module_name);
					$this->input_module->addOption($module_name, "{$module_info["displayName"]} (".blmsg($statuses[$module_info["status"]]).")");
				}
		}

		parent::initControl();
	}
	
	private function isHSPCAvailable()
	{
		if (is_null($this->getValue("hspcAvailable"))) {
			require_once("HSPC/Tools.php");
			$status = HSPCTools::getConnectionStatus();
			$this->setValue("hspcAvailable", $status["available"]);
		}
		return $this->getValue("hspcAvailable");
	}

	private $input_gateway_type;
	private $input_module;
}

class PaymentGatewayModuleParamsPage extends UIPage
{
	public function createControls()
	{
		$payment_gateway_module = $this->getValue("payment_gateway_module");
		if ( $this->getValue("selected_module") == $payment_gateway_module ) {
			$this->addModuleControls($payment_gateway_module);
		}
	}

	public function initControl()
	{
		$payment_gateway_module = $this->getValue("payment_gateway_module");
		if ( $this->getValue("selected_module") != $payment_gateway_module ) {
			$this->addModuleControls($payment_gateway_module);
			$this->setValue("selected_module", $payment_gateway_module);
		}

		parent::initControl();
	}

	public function processInput()
	{
		if ($this->isHSPCModuleSelected())
			return $this->processHSPCInput();

		$gateway = Factory()->PaymentGateway();
		$gateway->initFromArray($this->getValues());
		foreach ( $this->module_input_controls as $control ) {
			$gateway->PaymentGatewayParams[$control->name]->value = $control->value;
		}
		require_once("object_managers/PaymentGatewayManager.php");
		PaymentGatewayManager::update($gateway);
		return true;
	}

	private function addModuleControls($payment_gateway_module)
	{
		if ($this->isHSPCModuleSelected())
			return $this->addHSPCModuleForm($payment_gateway_module);

		$this->module_input_controls = array();
		require_once("ModuleTool.php");
		foreach ( ModuleTool::getModuleControls("gateway", $payment_gateway_module) as $control ) {
			$this->module_input_controls = array_merge($this->module_input_controls, $control->getInputControls());
			if ( "" == $control->title ) {
				$control->title = blmsg("TRANS_PAYMENT_GATEWAY_SETTINGS");
			}
			$this->addInputPanel($control);
		}
	}

	private function isHSPCModuleSelected()
	{
		require_once("objects/PaymentGateway.php");
		return in_array($this->getValue("payment_gateway_type"), array(PaymentGateway::TYPE_HSPC_CREDIT_CARD, PaymentGateway::TYPE_HSPC_THIRD_PARTY, PaymentGateway::TYPE_HSPC_BANK_TRANSFER));
	}

	private function processHSPCInput()
	{
		require_once("HSPC/PaymentProcess.php");

		$module_type = $this->getValue("payment_gateway_type");
		$module_name = $this->getValue("payment_gateway_module");

		$gateway = Factory()->PaymentGateway();
		$gateway->initFromArray($this->getValues());

		require_once("ModuleTool.php");
		$error = ModuleTool::getModuleInstance("gateway", "HSPCGateway", array("gateway" => $gateway))
				 ->collectPluginConfig();

		if ("" != $error)
		{
			$this->addErrorMessage(UIFactory()->ErrorMessage($error));
			return false;
		}

		require_once("object_managers/PaymentGatewayManager.php");
		PaymentGatewayManager::update($gateway);
		return true;
	}

	private function addHSPCModuleForm($module_name)
	{
		require_once("UI/Panel/UIHSPCInputPanel.php");
		require_once("ModuleTool.php");
		require_once("HSPC/Tools.php");

		$gateway = Factory()->PaymentGateway();
		$gateway->payment_gateway_type = $this->getValue("payment_gateway_type");
		$gateway->payment_gateway_module = $module_name;

		$form = ModuleTool::getModuleInstance("gateway", "HSPCGateway", array("gateway" => $gateway))
				->getPluginConfigForm();

		$this->getDefaultForm()->id = "form";
		$this->getDefaultForm()->name = "form";
		$this->getDefaultForm()->setMultipartEncryption();

		$this->addInputPanel(
			new UIHSPCInputPanel(blmsg("TRANS_HSPC_GATEWAY_SETTINGS"),
			HSPCTools::stylizePluginForm($form->form_content))
		);
	}

	private $module_input_controls;
}


class AddPaymentGateway extends MB_Action
{
	public function execute()
	{
		parent::execute();

		$back_url = action_url("paymentgateways", "ShowPaymentGateways");
		$controller  = UIFactory()->WizardController($back_url);
		$controller->addPage($select_module_page = new PaymentGatewaySelectModulePage($controller));
		$controller->addPage($module_params_page = new PaymentGatewayModuleParamsPage($controller));
		$select_module_page->title = $module_params_page->title = blmsg("TRANS_ADD_NEW_PAYMENT_GATEWAY");
		$select_module_page->parent_title = $module_params_page->parent_title = blmsg("TRANS_PAYMENT_GATEWAYS");
		$select_module_page->parent_url = $module_params_page->parent_url = $back_url;
		$controller->processRequest();

		$this->smartyVars["page"] = $controller->getPage();
		$this->addTemplate("page");
		return $this->hasExecutedSuccessfully();
	}
}
